from rest_framework.permissions import BasePermission

from . import verifiers


class BaseUserPermissions(verifiers.AbstractUserVerify, BasePermission):
    def has_permission(self, request, view):
        return self.verify(request.user)


class IsAdminSuperUserPerm(BaseUserPermissions, verifiers.IsAdminSuperUserVerify):
    """Check Admin Super User Permission"""


class IsCompanySuperUserPerm(BaseUserPermissions, verifiers.IsCompanySuperUserVerify):
    """Check Admin Super User Permission"""


class IsAdminScopeUserPerm(BaseUserPermissions, verifiers.AdminScopeUserVerify):
    """Check Admin User"""


class IsCompanyScopeUserPerm(BaseUserPermissions, verifiers.CompanyScopeUserVerify):
    """Check Company User"""


class IsCustomerScopeUserPerm(BaseUserPermissions, verifiers.CustomerScopeUserVerify):
    """Check Customer User"""


class CompanyOwnerPerm(BaseUserPermissions, verifiers.CompanyOwnerVerify):
    """Check Company Owner"""


class CompanyEmployeePerm(BaseUserPermissions, verifiers.CompanyEmployeeVerify):
    """Check Company Employee"""



import json

import redis
from django.conf import settings


class RedisAccessClient:
    _prefix = settings.REDIS_ACCESS_PREFIX

    def __init__(self):
        db = settings.REDIS_ACCESS_DB
        host = settings.REDIS_HOST
        port = settings.REDIS_PORT
        password = settings.REDIS_PASSWORD
        self.__redis_client = redis.StrictRedis(host=host, port=port, db=db, password=password)

    @property
    def conn(self):
        return self.__redis_client

    def __get(self, name: str) -> dict | None:
        value = self.conn.get(name)
        if value:
            return json.loads(value)

    def is_token_exist(self, token: str, prefix: bool = True) -> dict | None:
        key = f'{self._prefix}:{token}' if prefix else token
        return self.__get(key)



from abc import ABCMeta

from rest_framework.exceptions import ValidationError

from .verifiers import AbstractUserVerify, IsActiveUserVerify, NotBannedUserVerify
from ..models import BaseUser


class AbstractUserValidation(AbstractUserVerify, metaclass=ABCMeta):
    """Common abstract class for user validation"""

    exception: ValidationError

    def validate(self, user: BaseUser):
        """Raise error if user has not passed verification"""
        raise NotImplementedError


class BaseUserValidation(AbstractUserValidation):
    exception = ValidationError
    error_message = 'ValidationError'

    def __init__(self, message=''):
        if message:
            self.error_message = message

    def verify(self, user: BaseUser) -> bool:
        raise NotImplementedError

    def validate(self, user):
        if not self.verify(user):
            raise self.exception(self.error_message)


class ActiveUserValidator(IsActiveUserVerify, BaseUserValidation):
    """User Active Verification"""


class BannedUserValidatorVerify(NotBannedUserVerify, BaseUserValidation):
    """Banned User Verification"""



from ..models import CustomerUser, DeveloperUser, AdminUser, BaseUser


class AbstractUserVerify:
    """Common abstract class for user verification"""

    def verify(self, user: BaseUser) -> bool:
        """check user verification by condition"""
        raise NotImplementedError


class IsActiveUserVerify(AbstractUserVerify):
    def verify(self, user):
        return user.is_active


class NotBannedUserVerify(AbstractUserVerify):
    def verify(self, user):
        return not user.is_banned


class IsAdminSuperUserVerify(AbstractUserVerify):
    def verify(self, user):
        if isinstance(user, AdminUser):
            return user.is_superuser
        return False


class IsCompanySuperUserVerify(AbstractUserVerify):
    def verify(self, user):
        if isinstance(user, DeveloperUser):
            return user.is_superuser
        return False


class AdminScopeUserVerify(AbstractUserVerify):
    def verify(self, user) -> bool:
        return isinstance(user, AdminUser)


class CompanyScopeUserVerify(AbstractUserVerify):
    def verify(self, user) -> bool:
        return isinstance(user, DeveloperUser)


class CustomerScopeUserVerify(AbstractUserVerify):
    def verify(self, user) -> bool:
        return isinstance(user, CustomerUser)


class CompanyOwnerVerify(AbstractUserVerify):
    def verify(self, user) -> bool:
        if isinstance(user, DeveloperUser):
            return user.is_superuser
        return False


class CompanyEmployeeVerify(AbstractUserVerify):
    def verify(self, user):
        if isinstance(user, DeveloperUser):
            if user.company:
                return user.company.get('created_by') != user
        return False



from .redis_client import RedisAccessClient


class Token:
    redis_client: RedisAccessClient = RedisAccessClient()

    def get_token_data(self, token: str) -> dict:
        data = self.redis_client.is_token_exist(token)
        return data
